unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons,
  DSound, afDXSRec, afDXSUtil;

type
  TfrmMain = class(TForm)
    MemoStatus: TMemo;
    btnSelectWave: TButton;
    btnRecord: TButton;
    btnStop: TButton;
    btnClose: TButton;
    Label1: TLabel;
    SaveDialog1: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnSelectWaveClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnRecordClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
  private
    { Private declarations }
    Channels : integer;
    Bits : integer;
    SamplesPerSecond : longint;

    WaveFile : TafWAVEStream;
    Buff : TafDXSRecStreamingBuffer;
    procedure DataAvailable(Sender : TObject);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  // Set up the Capture object
  DXSRec_Init(Self);
  if Assigned(DXSRec) and DXSRec.Ready then MemoStatus.Lines.Add('DX Sound Capture available.')
    else MemoStatus.Lines.Add('DX Sound Capture not available.');
  // initialize the other objects we require (file and buffer)
  WaveFile := nil;
  Buff := nil;

  // I'll set the capture format here, rather than let the user set
  Channels := 1;
  Bits := 16;
  SamplesPerSecond := 22050;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  btnClose.Click;  // be sure file and buffer objects are freed
  DXSRec_Free;     // Free the Capture object
end;

procedure TfrmMain.DataAvailable(Sender : TObject);
var
  Ptr : Pointer;
  Size : Longint;
begin
  // This is called when the capture buffer has data for us...
  // It will be called every time the buffer adds audio data to it's
  // internal queue and every time we remove data if some data remains
  // for us to take

  Ptr := nil;
  Size := 0;     // note I'm not allocating the memory here... I'm
                 // passing nil and letting the function allocate the memory.

  Buff.RetrieveData(Ptr, Size);

  MemoStatus.Lines.Add('Writing to file (' + IntToStr(Size) + ' bytes)');
  if Assigned(WaveFile) then begin
    // write the next set of wave data to the file
    WaveFile.Write(Ptr^, Size);
    // note the pointer dereference above... this is necessary when
    // writing to a buffer specified by a pointer.
  end;
  FreeMem(Ptr, Size);
  // free the memory we used to hold the data from the capture buffer
end;

procedure TfrmMain.btnSelectWaveClick(Sender: TObject);
begin
  if SaveDialog1.Execute then
    if Assigned(DXSRec) and DXSRec.Ready then begin
      // if the DX Sound Capture Object was created
      // if a wave file is open, close it
      if Assigned(WaveFile) then WaveFile.Free;
      // open the wave file for writing (takes same args as TFileStream)
      WaveFile := TafWAVEStream.Create(SaveDialog1.Filename,
                                       fmCreate + fmShareCompat);
      // We need to tell the file the format of data we'll put into it
      WaveFile.Channels := Channels;
      WaveFile.Bits := Bits;
      WaveFile.SamplesPerSecond := SamplesPerSecond;

      // if the buffer is open, then free it
      if Assigned(Buff) then Buff.Free;
      // create a buffer to capture the wave (note buffer total size is 20000)
      // the streaming buffer size must ALWAYS be divisible by 2 !!!!
      Buff := DXSRec.NewStreamingBuffer(Channels,
                                        Bits,
                                        SamplesPerSecond,
                                        0,
                                        20000);
      // Tell the buffer where to notify us of available data
      Buff.OnDataAvailable := DataAvailable;
      MemoStatus.Lines.Add('Capture Buffer Created.');
      MemoStatus.Lines.Add('Wave File (' + SaveDialog1.Filename + ') Opened.');
    end else begin
      MemoStatus.Lines.Add('Capture not enabled.');
    end;
end;

procedure TfrmMain.btnRecordClick(Sender: TObject);
begin
  if Assigned(Buff) then begin
    // if the capture buffer exists, then...
    if Assigned(WaveFile) then WaveFile.Position := 0;
    // set the wave file position to the start of the file
    Buff.RecLoop;       // start capture
    MemoStatus.Lines.Add('Capture Started.');
  end;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
begin
  if Assigned(Buff) then begin
    // if the capture buffer exists, then...
    Buff.Stop;        // stop the buffer playback
    MemoStatus.Lines.Add('Capture Stopped.');
    // set the wave file position to the start of the file
    if Assigned(WaveFile) then WaveFile.Position := 0;
    MemoStatus.Lines.Add('Wave File reset to Start.');
  end;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  if Assigned(Buff) then begin
    // if the capture buffer exists, then...
    Buff.Free;    // free the buffer
    Buff := nil;  // reset its pointer to nil so we know it's been freed!
    MemoStatus.Lines.Add('Capture Buffer Destroyed.');
  end;
  if Assigned(WaveFile) then begin
    // if the wave file exists, then...
    WaveFile.Free;    // free the wave file
    WaveFile := nil;  // reset its pointer to nil so we know it's been freed!
    MemoStatus.Lines.Add('Wave File Closed.');
  end;
end;


end.
